@@ -16,7 +16,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- * $Id: bacon-video-widget-xine.c,v 1.181 2005/04/05 18:07:59 hadess Exp $
+ * $Id: bacon-video-widget-xine.c,v 1.188 2005/05/22 22:44:02 hadess Exp $
  *
  * the xine engine in a widget - implementation
  */
@@ -51,8 +51,8 @@
 #include "video-utils.h"
 #include "bacon-resize.h"
 
-#define DEFAULT_HEIGHT 315
-#define DEFAULT_WIDTH 420
+#define DEFAULT_HEIGHT 180
+#define DEFAULT_WIDTH 240
 #define CONFIG_FILE ".gnome2"G_DIR_SEPARATOR_S"totem_config"
 
 /* Signals */
@@ -238,6 +238,8 @@ static gboolean bacon_video_widget_motion_notify (GtkWidget *widget,
 		GdkEventMotion *event);
 static gboolean bacon_video_widget_button_press (GtkWidget *widget,
 		GdkEventButton *event);
+static void bacon_video_widget_show (GtkWidget *widget);
+static void bacon_video_widget_hide (GtkWidget *widget);
 
 static void bacon_video_widget_size_request (GtkWidget *widget,
 		GtkRequisition *requisition);
@@ -279,6 +281,8 @@ bacon_video_widget_class_init (BaconVideoWidgetClass *klass)
 	widget_class->expose_event = bacon_video_widget_expose;
 	widget_class->motion_notify_event = bacon_video_widget_motion_notify;
 	widget_class->button_press_event = bacon_video_widget_button_press;
+	widget_class->show = bacon_video_widget_show;
+	widget_class->hide = bacon_video_widget_hide;
 
 	/* GObject */
 	object_class->set_property = bacon_video_widget_set_property;
@@ -551,7 +555,7 @@ load_video_out_driver (BaconVideoWidget *bvw, gboolean null_out)
 	x11_visual_t vis;
 	const char *video_driver_id;
 	xine_video_port_t *vo_driver;
-	static char *drivers[] = { "xxmc", "xv", "xshm" };
+	static char *drivers[] = { "xv", "xshm" };
 	guint i;
 
 	if (null_out != FALSE)
@@ -1311,7 +1315,7 @@ xine_event_message (BaconVideoWidget *bvw, xine_ui_message_data_t *data)
 	}
 
 	sigdata = g_new0 (signal_data, 1);
-	sigdata->signal = MESSAGE_ASYNC;
+	sigdata->signal = ERROR_ASYNC;
 	sigdata->msg = message;
 	sigdata->num = num;
 	g_async_queue_push (bvw->priv->queue, sigdata);
@@ -1414,6 +1418,7 @@ xine_error (BaconVideoWidget *bvw, GError **error)
 	while ((data = g_async_queue_try_pop (bvw->priv->queue)) != NULL)
 	{
 		g_assert (data->signal == MESSAGE_ASYNC
+				|| data->signal == ERROR_ASYNC
 				|| data->signal == BUFFERING_ASYNC
 				|| data->signal == REDIRECT_ASYNC
 				|| data->signal == EOS_ASYNC);
@@ -1653,6 +1658,8 @@ bacon_video_widget_new (int width, int height,
 			bvw->priv->ao_driver, bvw->priv->vo_driver);
 	setup_config_stream (bvw);
 	bvw->priv->ev_queue = xine_event_new_queue (bvw->priv->stream);
+	xine_event_create_listener_thread (bvw->priv->ev_queue,
+			xine_event, (void *) bvw);
 
 	return GTK_WIDGET (bvw);
 }
@@ -1703,6 +1710,28 @@ bacon_video_widget_button_press (GtkWidget *widget, GdkEventButton *event)
 	return FALSE;
 }
 
+static void
+bacon_video_widget_show (GtkWidget *widget)
+{
+	BaconVideoWidget *bvw = (BaconVideoWidget *) widget;
+
+	gdk_window_show (bvw->priv->video_window);
+
+	if (GTK_WIDGET_CLASS (parent_class)->show != NULL)
+		(* GTK_WIDGET_CLASS (parent_class)->show) (widget);
+}
+
+static void
+bacon_video_widget_hide (GtkWidget *widget)
+{
+	BaconVideoWidget *bvw = (BaconVideoWidget *) widget;
+
+	gdk_window_hide (bvw->priv->video_window);
+
+	if (GTK_WIDGET_CLASS (parent_class)->hide != NULL)
+		(* GTK_WIDGET_CLASS (parent_class)->hide) (widget);
+}
+
 static void
 bacon_video_widget_size_request (GtkWidget *widget, GtkRequisition *requisition)
 {
@@ -2930,7 +2959,7 @@ bacon_video_widget_can_play (BaconVideoWidget *bvw, MediaType type)
 	}
 }
 
-G_CONST_RETURN char
+char
 **bacon_video_widget_get_mrls (BaconVideoWidget *bvw, MediaType type)
 {
 	char *plugin_id;
@@ -2949,8 +2978,8 @@ G_CONST_RETURN char
 	else
 		return NULL;
 
-	return (G_CONST_RETURN char **) xine_get_autoplay_mrls
-		(bvw->priv->xine, plugin_id, &num_mrls);
+	return g_strdupv (xine_get_autoplay_mrls
+		(bvw->priv->xine, plugin_id, &num_mrls));
 }
 
 void
@@ -3068,6 +3097,7 @@ bacon_video_widget_set_zoom (BaconVideoWidget *bvw, int zoom)
 	g_return_if_fail (bvw != NULL);
 	g_return_if_fail (BACON_IS_VIDEO_WIDGET (bvw));
 	g_return_if_fail (bvw->priv->xine != NULL);
+	g_return_if_fail (zoom >= 0 && zoom <= 400);
 
 	xine_set_param (bvw->priv->stream,
 			XINE_PARAM_VO_ZOOM_X, zoom);
@@ -3446,7 +3476,7 @@ GList
 *bacon_video_widget_get_subtitles (BaconVideoWidget *bvw)
 {
 	GList *list = NULL;
-	int i;
+	int i, num_channels;
 	char lang[XINE_LANG_MAX];
 
 	g_return_val_if_fail (BACON_IS_VIDEO_WIDGET (bvw), NULL);
@@ -3455,7 +3485,13 @@ GList
 	if (bvw->priv->mrl == NULL)
 		return NULL;
 
-	for(i = 0; i < 32; i++)
+	num_channels = xine_get_stream_info
+		(bvw->priv->stream, XINE_STREAM_INFO_MAX_SPU_CHANNEL);
+
+	if (num_channels < 2)
+		return NULL;
+
+	for(i = 0; i < num_channels; i++)
 	{
 		memset (&lang, 0, sizeof (lang));
 
@@ -3468,6 +3504,11 @@ GList
 
 			list = g_list_prepend (list,
 					(gpointer) g_strdup (nospace));
+		} else {
+			/* An unnamed language, for example 'Language 2' */
+			list = g_list_prepend (list,
+					(gpointer) g_strdup_printf
+					(_("Language %d"), i + 1));
 		}
 	}
 
@@ -3545,7 +3586,7 @@ bacon_video_widget_get_current_frame (BaconVideoWidget *bvw)
 	if (width == 0 || height == 0)
 		return NULL;
 
-	yuv = malloc ((width + 8) * (height + 1) * 2);
+	yuv = g_malloc ((width + 8) * (height + 1) * 2);
 	if (yuv == NULL)
 		return NULL;
 
@@ -3562,7 +3603,7 @@ bacon_video_widget_get_current_frame (BaconVideoWidget *bvw)
 		{
 			uint8_t *yuy2 = yuv;
 
-			yuv = malloc (width * height * 2);
+			yuv = g_malloc (width * height * 2);
 			y = yuv;
 			u = yuv + width * height;
 			v = yuv + width * height * 5 / 4;
