@@ -16,6 +16,15 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
+ *
+ * The Totem project hereby grant permission for non-gpl compatible GStreamer
+ * plugins to be used and distributed together with GStreamer and Totem. This
+ * permission are above and beyond the permissions granted by the GPL license
+ * Totem is covered by.
+ *
+ * Monday 7th February 2005: Christian Schaller: Add excemption clause.
+ * See license_change file for details.
+ *
  */
 
 #include <config.h>
@@ -25,7 +34,6 @@
 #include <stdio.h>
 #include <sys/stat.h>
 #include <sys/types.h>
-#include <nautilus-burn-drive-selection.h>
 
 #include "totem.h"
 #include "totem-private.h"
@@ -132,7 +140,7 @@ totem_prefs_set_show_visuals (Totem *totem, gboolean value, gboolean warn)
 	{
 		totem_action_info (_("The change of this setting will only "
 					"take effect for the next movie, or "
-					"when Totem is restarted"),
+					"when Totem is restarted."),
 				totem);
 	}
 }
@@ -206,17 +214,6 @@ deinterlace_changed_cb (GConfClient *client, guint cnxn_id,
 			G_CALLBACK (on_deinterlace1_activate), totem);
 }
 
-static void
-on_combo_entry1_changed (NautilusBurnDriveSelection *bcs, char *device, Totem *totem)
-{
-	const char *str;
-
-	str = nautilus_burn_drive_selection_get_device (bcs);
-	gconf_client_set_string (totem->gc, GCONF_PREFIX"/mediadev", str, NULL);
-	bacon_video_widget_set_media_device
-		(BACON_VIDEO_WIDGET (totem->bvw), str);
-}
-
 static void
 auto_resize_changed_cb (GConfClient *client, guint cnxn_id,
 		GConfEntry *entry, Totem *totem)
@@ -254,28 +251,17 @@ show_vfx_changed_cb (GConfClient *client, guint cnxn_id,
 }
 
 static void
-mediadev_changed_cb (GConfClient *client, guint cnxn_id,
+disable_save_to_disk_changed_cb (GConfClient *client, guint cnxn_id,
 		GConfEntry *entry, Totem *totem)
 {
 	GtkWidget *item;
-	char *mediadev;
+	gboolean locked;
 
-	item = glade_xml_get_widget (totem->xml, "tpw_device_combo");
-	g_signal_handlers_disconnect_by_func (G_OBJECT (item),
-			on_combo_entry1_changed, totem);
-
-	mediadev = gconf_client_get_string (totem->gc,
-			GCONF_PREFIX"/mediadev", NULL);
-
-	if (mediadev == NULL || strcmp (mediadev, "") == 0)
-		mediadev = g_strdup ("/dev/cdrom");
-
-	nautilus_burn_drive_selection_set_device (NAUTILUS_BURN_DRIVE_SELECTION (item), mediadev);
-	bacon_video_widget_set_media_device
-		(BACON_VIDEO_WIDGET (totem->bvw), mediadev);
-
-	g_signal_connect (G_OBJECT (item), "device-changed",
-			G_CALLBACK (on_combo_entry1_changed), totem);
+	locked = gconf_client_get_bool (totem->gc,
+			"/desktop/gnome/lockdown/disable_save_to_disk", NULL);
+	item = glade_xml_get_widget (totem->xml,
+			"tmw_take_screenshot_menu_item");
+	gtk_widget_set_sensitive (item, !locked);
 }
 
 static void
@@ -288,38 +274,11 @@ connection_combobox_changed (GtkComboBox *combobox, Totem *totem)
 		(BACON_VIDEO_WIDGET (totem->bvw), i);
 }
 
-static void
-on_button1_clicked (GtkButton *button, Totem *totem)
-{
-	GError *err = NULL;
-	char *path, *cmd, *filemanager;
-
-	path = g_build_path (G_DIR_SEPARATOR_S,
-			g_get_home_dir (), PROPRIETARY_PLUGINS, NULL);
-
-	if ((g_getenv ("KDE_STARTUP_ENV") || g_getenv("KDE_FULL_SESSION")) && (filemanager = g_find_program_in_path("konqueror"))) {
-		cmd = g_strconcat (filemanager, " ", path, NULL);
-		g_free (filemanager);
-	} else {
-		cmd = g_strdup_printf ("nautilus --no-default-window -no-desktop %s", path);
-	}
-	g_free (path);
-
-	if (g_spawn_command_line_async (cmd, &err) == FALSE)
-	{
-		totem_action_error (_("Totem could not start the file manager."), err->message, totem);
-		g_error_free (err);
-	}
-
-	g_free (cmd);
-}
-
 static void
 visual_menu_changed (GtkComboBox *combobox, Totem *totem)
 {
 	GList *list;
-	const char *old_name;
-	char *name;
+	char *old_name, *name;
 	int i;
 
 	i = gtk_combo_box_get_active (combobox);
@@ -337,6 +296,8 @@ visual_menu_changed (GtkComboBox *combobox, Totem *totem)
 		if (bacon_video_widget_set_visuals (totem->bvw, name) != FALSE)
 			totem_action_info (_("Changing the visuals effect type will require a restart to take effect."), totem);
 	}
+
+	g_free (old_name);
 }
 
 static void
@@ -350,17 +311,6 @@ visual_quality_menu_changed (GtkComboBox *combobox, Totem *totem)
 	bacon_video_widget_set_visuals_quality (totem->bvw, i);
 }
 
-GtkWidget *
-bacon_cd_selection_create (void)
-{
-	GtkWidget *widget;
-
-	widget = nautilus_burn_drive_selection_new ();
-	gtk_widget_show (widget);
-
-	return widget;
-}
-
 static void
 brightness_changed (GtkRange *range, Totem *totem)
 {
@@ -423,9 +373,40 @@ static void
 audio_out_menu_changed (GtkComboBox *combobox, Totem *totem)
 {
 	BaconVideoWidgetAudioOutType audio_out;
+	gboolean need_restart;
 
 	audio_out = gtk_combo_box_get_active (combobox);
-	bacon_video_widget_set_audio_out_type (totem->bvw, audio_out);
+	need_restart = bacon_video_widget_set_audio_out_type (totem->bvw, audio_out);
+	if (need_restart != FALSE) {
+		totem_action_info (_("The change of audio output type will "
+					"only take effect when Totem is "
+					"restarted."),
+				totem);
+	}
+
+}
+
+static void
+on_font_set (GtkFontButton * fb, Totem * totem)
+{
+	const gchar *font;
+
+	font = gtk_font_button_get_font_name (fb);
+	gconf_client_set_string (totem->gc, GCONF_PREFIX"/subtitle_font",
+				 font, NULL);
+}
+
+static void
+font_changed_cb (GConfClient *client, guint cnxn_id,
+		 GConfEntry *entry, Totem *totem)
+{
+	const gchar *font;
+	GtkWidget *item;
+
+	item = glade_xml_get_widget (totem->xml, "font_sel_button");
+	font = gconf_value_get_string (entry->value);
+	gtk_font_button_set_font_name (GTK_FONT_BUTTON (item), font);
+	bacon_video_widget_set_subtitle_font (totem->bvw, font);
 }
 
 void
@@ -434,7 +415,7 @@ totem_setup_preferences (Totem *totem)
 	GtkWidget *item, *menu;
 	gboolean show_visuals, auto_resize, is_local, deinterlace;
 	int connection_speed, i;
-	char *path, *visual, *mediadev;
+	char *path, *visual, *font;
 	GList *list, *l;
 	BaconVideoWidgetAudioOutType audio_out;
 
@@ -450,8 +431,12 @@ totem_setup_preferences (Totem *totem)
 	gconf_client_notify_add (totem->gc, GCONF_PREFIX"/show_vfx",
 			(GConfClientNotifyFunc) show_vfx_changed_cb,
 			totem, NULL, NULL);
-	gconf_client_notify_add (totem->gc, GCONF_PREFIX"/mediadev",
-			(GConfClientNotifyFunc) mediadev_changed_cb,
+	gconf_client_add_dir (totem->gc, "/desktop/gnome/lockdown",
+			GCONF_CLIENT_PRELOAD_ONELEVEL, NULL);
+	gconf_client_notify_add (totem->gc,
+			"/desktop/gnome/lockdown/disable_save_to_disk",
+			(GConfClientNotifyFunc)
+			disable_save_to_disk_changed_cb,
 			totem, NULL, NULL);
 
 	totem->prefs = glade_xml_get_widget (totem->xml,
@@ -472,29 +457,6 @@ totem_setup_preferences (Totem *totem)
 	bacon_video_widget_set_auto_resize
 		(BACON_VIDEO_WIDGET (totem->bvw), auto_resize);
 
-	/* Media device */
-	item = glade_xml_get_widget (totem->xml, "tpw_device_combo");
-	mediadev = g_strdup (gconf_client_get_string
-		(totem->gc, GCONF_PREFIX"/mediadev", NULL));
-	if (mediadev == NULL || (strcmp (mediadev, "") == 0)
-			|| (strcmp (mediadev, "auto") == 0))
-	{
-		mediadev = g_strdup
-			(nautilus_burn_drive_selection_get_default_device
-				     (NAUTILUS_BURN_DRIVE_SELECTION (item)));
-		gconf_client_set_string (totem->gc, GCONF_PREFIX"/mediadev",
-				mediadev, NULL);
-	}
-	bacon_video_widget_set_media_device
-		(BACON_VIDEO_WIDGET (totem->bvw), mediadev);
-
-	g_signal_connect (G_OBJECT (item), "device-changed",
-			G_CALLBACK (on_combo_entry1_changed), totem);
-
-	nautilus_burn_drive_selection_set_device
-		(NAUTILUS_BURN_DRIVE_SELECTION (item), mediadev);
-	g_free (mediadev);
-
 	/* Connection Speed */
 	connection_speed = bacon_video_widget_get_connection_speed (totem->bvw);
 	item = glade_xml_get_widget (totem->xml, "tpw_speed_combobox");
@@ -503,9 +465,6 @@ totem_setup_preferences (Totem *totem)
 			G_CALLBACK (connection_combobox_changed), totem);
 
 	/* Proprietary plugins */
-	item = glade_xml_get_widget (totem->xml, "tpw_plugins_button");
-	g_signal_connect (G_OBJECT (item), "clicked",
-			G_CALLBACK (on_button1_clicked), totem);
 	path = g_build_path (G_DIR_SEPARATOR_S,
 			g_get_home_dir (), PROPRIETARY_PLUGINS, NULL);
 	if (g_file_test (path, G_FILE_TEST_IS_DIR) == FALSE)
@@ -533,8 +492,10 @@ totem_setup_preferences (Totem *totem)
 
 	visual = gconf_client_get_string (totem->gc,
 			GCONF_PREFIX"/visual", NULL);
-	if (visual == NULL || strcmp (visual, "") == 0)
+	if (visual == NULL || strcmp (visual, "") == 0) {
+		g_free (visual);
 		visual = g_strdup ("goom");
+	}
 
 	item = glade_xml_get_widget (totem->xml, "tpw_visuals_type_combobox");
 	g_signal_connect (G_OBJECT (item), "changed",
@@ -626,6 +587,30 @@ totem_setup_preferences (Totem *totem)
 	gtk_check_menu_item_set_active (GTK_CHECK_MENU_ITEM (item),
 			gconf_client_get_bool (totem->gc,
 				GCONF_PREFIX"/window_on_top", NULL));
+
+	/* Save to disk Lockdown */
+	item = glade_xml_get_widget
+		(totem->xml, "tmw_take_screenshot_menu_item");
+	gtk_widget_set_sensitive (item,
+			!gconf_client_get_bool (totem->gc,
+				"/desktop/gnome/lockdown/disable_save_to_disk",
+				NULL));
+
+	/* subtitle font selection */
+	item = glade_xml_get_widget (totem->xml, "font_sel_button");
+	g_signal_connect (item, "font-set", G_CALLBACK (on_font_set), totem);
+	gtk_font_button_set_title (GTK_FONT_BUTTON (item),
+				   _("Select Subtitle Font"));
+	font = gconf_client_get_string (totem->gc,
+		GCONF_PREFIX"/subtitle_font", NULL);
+	if (font && strcmp (font, "") != 0) {
+		gtk_font_button_set_font_name (GTK_FONT_BUTTON (item), font);
+		bacon_video_widget_set_subtitle_font (totem->bvw, font);
+	}
+	g_free (font);
+	gconf_client_notify_add (totem->gc, GCONF_PREFIX"/subtitle_font",
+			(GConfClientNotifyFunc) font_changed_cb,
+			totem, NULL, NULL);
 }
 
 void
@@ -698,8 +683,10 @@ totem_preferences_visuals_setup (Totem *totem)
 
 	visual = gconf_client_get_string (totem->gc,
 			GCONF_PREFIX"/visual", NULL);
-	if (visual == NULL || strcmp (visual, "") == 0)
+	if (visual == NULL || strcmp (visual, "") == 0) {
+		g_free (visual);
 		visual = g_strdup ("goom");
+	}
 
 	bacon_video_widget_set_visuals (totem->bvw, visual);
 	g_free (visual);
